
#include "perm/rotate.h"
#include "fxttypes.h"

void
ffact2perm_rot(const ulong *fc, ulong n, ulong *x)
// Convert the (n-1) digit falling factorial representation in fc[0,...,n-2].
// into permutation in x[0,...,n-1]
// Must have: fc[0]<n, fc[1]<n-1, ..., fc[n-2]<2 (falling radices)
{
    for (ulong k=0; k<n; ++k)  x[k] = k;
    for (ulong k=0, len=n;  k<n-1;  ++k, --len)
    {
        ulong fa = fc[k];
        rotate_left(x+k, len, fa);
    }
}
// -------------------------

void
rfact2perm_rot(const ulong *fc, ulong n, ulong *x)
// Convert the (n-1) digit rising factorial representation in fc[0,...,n-2].
//  into permutation in x[0,...,n-1]
// Must have: fc[0]<2, fc[1]<3, ..., fc[n-2]<n (rising radices)
{
    for (ulong k=0; k<n; ++k)  x[k] = k;
    for (ulong k=n-2, len=n;  (long)k>=0;  --k, --len)
    {
        ulong fa = fc[k];
        rotate_left(x+n-len, len, fa);
    }
}
// -------------------------
